# pylint: disable=no-self-use
import logging

from flask import request
from flask_restplus import Namespace, Resource, inputs, abort
from flask_restplus.reqparse import RequestParser
from spectacular import FailedToValidate

from airflow import settings
from airflow.models import DAG
from airflow.www.app import csrf
from shu.interactions import Interactions, InteractionError, NotFound
from shu.wrappers import DagRunWrapper
from shu.adapters import create_adapters

dags = Namespace("dags")
log = logging.getLogger(__name__)

# Explanation of priorities (#A, #B, #C)
#  #A: Needs to be in prototype
#  #B: Needs to be in when deploying to production
#  #C: Better behaviour, but at this stage non-essential

# TODO: [#C] Ensure the DAG model also appears in the API documentation,
#            Add something like, marshall_with to DagCollection.get
# TODO: [#C] Remove 'message' field from Flask-RestPlus error response. See also:
#            https://github.com/noirbizarre/flask-restplus/pull/377


class Parser:
    @staticmethod
    def get_dag_collection():
        parser = RequestParser()
        parser.add_argument("prefix", type=str, required=False)
        parser.add_argument("is_active", type=inputs.boolean)
        return parser

    @staticmethod
    def get_dag_run_collection():
        parser = RequestParser()
        parser.add_argument("prefix", type=str, required=False)
        parser.add_argument("state", type=str, required=False)
        return parser


def interactions():
    adapters = create_adapters(settings.Session())
    return Interactions(adapters)


@dags.route("")
class DagCollection(Resource):
    @dags.expect(Parser.get_dag_collection())
    def get(self):
        """List DAGs"""
        try:
            arguments = Parser.get_dag_collection().parse_args()
            return interactions().find_dags(arguments).to_dict()
        except (InteractionError, FailedToValidate) as exc:
            abort(400, error=exc.to_dict())


@dags.route("/<string:dag_id>/runs")
class DagRunCollection(Resource):
    @dags.expect(dags.schema_model("DagRun", DagRunWrapper.schema(ignored=["dag_id"])))
    def post(self, dag_id):
        try:
            return interactions().create_dag_run(dag_id, request.json).to_dict()
        except (InteractionError, FailedToValidate) as exc:
            abort(400, error=exc.to_dict())

    @dags.expect(Parser.get_dag_run_collection())
    def get(self, dag_id):
        """List DAG runs"""
        try:
            return interactions().find_dag_run(dag_id, request.args).to_dict()
        except NotFound as exc:
            if exc.cls == DAG:
                abort(404, error=exc.to_dict())
        except (InteractionError, FailedToValidate) as exc:
            abort(400, error=exc.to_dict())


@dags.errorhandler
def default_error_handler(error):
    log.info(error)



import logging
import os
from functools import lru_cache
import flask

import pkg_resources
from airflow.www.app import csrf
from flask import Blueprint, url_for
from flask_restplus import Api, Namespace, Resource, apidoc
from flask_restplus.api import SwaggerView

log = logging.getLogger(__name__)


def transform_keys(m, **keys):
    return {k: f(m[k]) for (k, f) in keys.items() if k in m}


@lru_cache(maxsize=1)
def _get_version():
    try:
        return pkg_resources.get_distribution("shu").version or "unknown"
    except Exception as exc:  # pylint: disable=broad-except
        log.error("Could not determine version: %s", exc)
        return "unknown"


def create_api(name, **extra):
    # pylint: disable=unused-variable
    """Create an API with info and health endpoints.

    :param name: The name of the application; used in the generated frontend,
        and the URL prefix.
    :param extra: A dict of parameters passed directly to the
        :class:`~CustomAPI` constructor.

    :returns: An instance of :class:`~apian.api.CustomAPI`.
    """
    blueprint = Blueprint(name, __name__)
    api = CustomAPI(blueprint, title="api", prefix=name, doc=name + "/", **extra)
    ns = Namespace("Generic")

    @api.route("/health")
    class Health(Resource):
        def get(self):
            # pylint: disable=no-self-use
            """Check the health of the service"""
            return {"status": "ok"}

    keys = {
        "WTF_CSRF_ENABLED": lambda x: x,
        "WTF_CSRF_CHECK_DEFAULT": lambda x: x,
        "WTF_CSRF_METHODS": list,
    }

    @api.route("/info")
    class Info(Resource):
        def get(self):
            return {
                "csrf": {
                    "exempt": {
                        "blueprints": list(csrf._exempt_blueprints),
                        "views": list(csrf._exempt_views),
                    }
                },
                "name": "shu",
                "version": _get_version(),
                "request": {"blueprint": str(flask.request.blueprint)},
                "config": transform_keys(flask.current_app.config, **keys),
            }

    return api


def create_swagger():
    swagger = apidoc.Apidoc(
        "restplus_custom_doc",
        __name__,
        template_folder="templates",
        static_folder=os.path.dirname(apidoc.__file__) + "/static",
        static_url_path="/swagger",
    )

    @swagger.add_app_template_global
    def swagger_static(filename):
        # pylint: disable=unused-variable
        return url_for("restplus_custom_doc.static", filename=filename)

    return swagger


class CustomAPI(Api):
    """
    Hack to serve swagger.json from the /api prefix.  This is a requirement for
    serving swagger-ui from behind a path-routing load balancer without
    additional proxies.
    """

    def _register_specs(self, app_or_blueprint):
        if self._add_specs:
            endpoint = str("specs")
            self._register_view(
                app_or_blueprint,
                SwaggerView,
                self.default_namespace,
                "{}/swagger.json".format(self.prefix),
                endpoint=endpoint,
                resource_class_args=(self,),
            )
            self.endpoints.add(endpoint)

    @property
    def specs_url(self):
        """Swagger URI protocol-relative."""
        return url_for(self.endpoint("specs"), _external=True, _scheme="")
